#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals

__license__   = 'GPL v3'
__copyright__ = u'2019, Jürgen Habelt <juergen@habelt-jena.de>'
__docformat__ = 'restructuredtext en'

globals()['load_translations']()

from PyQt5.Qt import QDialog, QPushButton

try:
    from calibre_plugins.define_content.json_wrapper import JsonConfigWrapper 
    from calibre_plugins.define_content.gui.TempConfig_ui import Ui_TempConfig

except ImportError:
    from json_wrapper import JsonConfigWrapper 
    from gui.TempConfig_ui import Ui_TempConfig

# This is where all preferences for this plug-in will be stored
prefs = JsonConfigWrapper.getInstance()


class TempConfigMixin(object):
    '''
    Mixin of the code for the Temp Config Dialog
    '''
    
    def __init__(self):
        '''
        Initialization
        '''
        # setup the User Interface
        self.setupUi(self)
        
        # we extract all translatable text from the generated UI code
        self.set_text()
        
        # load the settings from configuration file
        self.load_settings()
    
    
    def set_text(self):
        '''
        Contains all translatable text
        '''
        self.levelLabel.setText(_("Max Level:"))
        self.levelSpinBox.setToolTip(_("The maximum level of Headings to be displayed in #content"))
        self.stripLabel.setText(_("Strip:"))
        self.useStripCheckBox.setToolTip(_("A flag indicating whether to strip some characters at the beginning of Headings"))
        self.stripLineEdit.setToolTip(_("A regular expression defining the characters to be stripped"))
        self.omitLabel.setText(_("Omit:"))
        self.omitCheckBox.setToolTip(_("A flag indicating whether to omit the list item numbers of second level Headings"))
        self.continuousLabel.setText(_("Continuous:"))
        self.continuousCheckBox.setToolTip(_("A flag indicating whether to count second level Headings continuously"))
        
        
    def load_settings(self):
        '''
        Initializes controls with settings
        '''
        self.levelSpinBox.setValue(self.settings['max_level'])
        self.useStripCheckBox.setChecked(self.settings['use_strip'])
        self.stripLineEdit.setText(self.settings['strip'])
        self.omitCheckBox.setChecked(self.settings['omit'])
        self.continuousCheckBox.setChecked(self.settings['continuous'])
        
        
    def save_settings(self):
        '''
        Saves the settings stored in the controls
        '''
        self.settings['max_level'] = self.levelSpinBox.value()
        self.settings['use_strip'] = self.useStripCheckBox.isChecked()
        self.settings['strip'] = self.stripLineEdit.text()
        self.settings['omit'] = self.omitCheckBox.isChecked()
        self.settings['continuous'] = self.continuousCheckBox.isChecked()


class TempConfigDialog(QDialog, Ui_TempConfig, TempConfigMixin):
    '''
    For temporary configuration during action
    '''
    
    def __init__(self, gui):
        '''
        Initialization, constructs the widget.
        '''
        QDialog.__init__(self, gui)
        self.settings = prefs.dup()

        TempConfigMixin.__init__(self)        
        
        self.setWindowTitle(_('Temporary Settings'))

        self.proceedPushButton = QPushButton(_('Proceed'), self)
        self.proceedPushButton.clicked.connect(self.proceed)
        self.proceedPushButton.setToolTip(_('Proceed with the Define Content action'))
        self.gridLayout_2.addWidget(self.proceedPushButton, 4, 2)
        
        
    def proceed(self):
        '''
        Continuous the action        
        '''
        self.save_settings()
        self.accept()
       
