#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals

__license__   = 'GPL v3'
__copyright__ = u'2019, Jürgen Habelt <juergen@habelt-jena.de>'
__docformat__ = 'restructuredtext en'

import os
from xml.etree import ElementTree
from zipfile import ZipFile

try:
    from calibre_plugins.define_content.logging_factory import LoggingFactory
    from calibre_plugins.define_content.ruby_port.bookmark import Bookmark
    from calibre_plugins.define_content.ruby_port.extractor import Extractor

except ImportError:
    from logging_factory import LoggingFactory
    from ruby_port.bookmark import Bookmark
    from ruby_port.extractor import Extractor


class EpubExtractor(Extractor):
    '''
    Class responsible for extracting and structuring book mark information
    '''
    
    def __init__(self, invoker):
        '''
        Initialization
        @param invoker: the injected Invoker is responsible for invoking other processes 
        '''
        self.invoker = invoker
    
    
    def extract(self, frm, result):
        '''
        Extracts the book mark info from EPUB file and writes it into a XML file
        @param frm: the EPUB file from which to extract the book marks
        '''
        path = os.path.dirname(result)
        
        with ZipFile(frm, 'r') as zip_file:
            potential_tocs = ['toc.ncx', 'OEBPS/toc.ncx', 'OEBPS/html/toc.ncx']
            for member in potential_tocs:
                try:
                    zip_file.extract(member, path)
                    return os.path.join(path, member)
                
                except Exception as ex:
                    pass
        
            raise ex
        
    
    def parse_node(self, bookmark, nav_point):
        '''
        Parses a xml element node recursively and adds children to the parent book mark
        '''
        for nav_point_child in nav_point.iterfind('epub_ns:navPoint', self.ns):
            LoggingFactory().getLogger(self).debug("navPoint tag: %s" % nav_point_child.tag)        
            title = nav_point_child.findtext('epub_ns:navLabel/epub_ns:text', 'no title', self.ns)
            if title == None:
                continue
            
            title = self.strip_from_title(title)                                                    # strip from the beginning
            LoggingFactory().getLogger(self).debug("title text: %s" % title)        
            child = Bookmark(title, bookmark.level + 1, bookmark)
            
            if bookmark.level == 1:
                self.level2_number += 1
                child.number = self.level2_number
            
            bookmark.children.append(child)
      
            self.parse_node(child, nav_point_child)
                
        
    def parse(self, frm):
        '''
        Parses a file into a book mark tree and returns the root node
        @param frm: the xml file from which to evaluate the book marks
        '''        
        root = Bookmark( "Root" )
        self.level2_number = 0

        with open(frm, 'r') as file:
            xml_text = file.read()

        self.ns = { 'epub_ns': "http://www.daisy.org/z3986/2005/ncx/" }
        xml_root = ElementTree.fromstring(xml_text)
        LoggingFactory().getLogger(self).debug("Root tag: %s" % xml_root.tag)        
        
        for nav_map in xml_root.iterfind('epub_ns:navMap', self.ns):
            LoggingFactory().getLogger(self).debug("navMap tag: %s" % nav_map.tag)        
            self.parse_node(root, nav_map)

        return root
        