#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from subprocess import CalledProcessError

__license__   = 'GPL v3'
__copyright__ = u'2019, Jürgen Habelt <juergen@habelt-jena.de>'
__docformat__ = 'restructuredtext en'

import re

try:
    from calibre_plugins.define_content.ruby_port.bookmark import Bookmark
    from calibre_plugins.define_content.ruby_port.extractor import Extractor

except ImportError:
    from ruby_port.bookmark import Bookmark
    from ruby_port.extractor import Extractor


class PdfExtractor(Extractor):
    '''
    Class responsible for extracting and structuring book mark information
    '''
    
    def __init__(self, invoker):
        '''
        Initialization
        @param invoker: the injected Invoker is responsible for invoking other processes 
        '''
        self.invoker = invoker
        
        
    def extract(self, frm, result):
        '''
        Extracts the book mark info from PDF file and writes it into a text file
        @param frm: the PDF file from which to extract the book marks
        '''
        to = self.change_extension(result, '.txt')
        ecode, output = self.invoker.invokePdftk(frm)
        
        if ecode != 0:
            raise CalledProcessError('Could not invoke PDFtk tool')

        with open(to, 'w') as f:
            f.write(output)

        return to
        
        
    def each_bookmark(self, frm):
        '''
        Extracts the book marks from a text file yielding book mark objects
        @param frm: the text file from which to evaluate the book marks
        '''
        with open(frm, 'r') as f:
            lines = f.readlines()
        lines = [line for line in lines if line.startswith('Bookmark')]
        for idx, line in enumerate(lines):
            if idx % 4 == 1:                                                            # Title
                title = re.search(r'^BookmarkTitle\: *(.*?)\r?\n$', line).group(1)      # extract from line
                title = self.strip_from_title( title )                                  # strip from the beginning
            if idx % 4 == 2:                                                            # Level
                level = int(re.search(r'^BookmarkLevel\: *(.*?)\r?\n$', line).group(1))
                bookmark = Bookmark(title, level)
                yield bookmark
                
        
    def parse(self, frm):
        '''
        Parses a file into a book mark tree and returns the root node
        @param frm: the text file from which to evaluate the book marks
        '''        
        root = Bookmark( "Root" )
        cur_parent = root
        prev_child = None
        level2_number = 0
        
        for bookmark in self.each_bookmark(frm):
          
            if bookmark.level == cur_parent.level + 2:                  # prepare book mark and its parent
                cur_parent = prev_child

            while bookmark.level <= cur_parent.level:
                cur_parent = cur_parent.parent
                                                                        # prepared !
            if bookmark.level == 2:                                     # at this point we have level 2 book mark 
                level2_number += 1                                      # increase counter
                if len(cur_parent.children) == 0:                       # it's the first child of parent
                    cur_parent.number = level2_number                   # set its number in parent !
          
            bookmark.parent = cur_parent                                # finally add to tree
            cur_parent.children.append(bookmark)
            prev_child = bookmark
    
        return root
        
        
        
        